@@ -19,6 +19,7 @@ import csv
 import datetime
 import inspect
 import logging
+import logging.config
 import re
 import sys
 import subprocess
@@ -29,6 +30,7 @@ from os import access, getenv, R_OK, makedirs, path, stat, system
 from deprecated import deprecated
 
 snow_debug_colors = getenv("SNOW_DEBUG_COLOR", "").strip().upper()
+snow_logging_enabled = getenv("SNOW_LOGGING", False)
 
 if snow_debug_colors:
     import termcolor
@@ -74,10 +76,43 @@ def configure_log():
     The log file is named 'snowflake_python_connector.log'
     """
     logging.basicConfig(
-        filename='snowflake_python_connector.log',
+        filename= 'snowflake_python_connector.log',
         level=logging.DEBUG,
+        format="{asctime} {levelname:<8} {message}",
+        style="{",
         filemode='w')
 
+def configure_log(configuration_path):
+    """Configures the logging that will be performed for any data related execution on the snowflake connection.
+
+        configuration_path
+            The configuration path of the file that contains all the settings desired for the logging.
+
+    """
+    logging.config.fileConfig(configuration_path)
+
+def log(*msg, level=logging.INFO, writter=None):
+    """ Prints a message to the console (standard output) or to the log file, depending on if logging is enabled.
+
+        msg
+            The message to print or log.
+    """
+    if snow_logging_enabled :
+        finalmsg = ""
+        for item in msg :
+            finalmsg += re.sub('\033\[\d*m','', str(item))
+        if level == logging.ERROR :
+            logging.error(finalmsg)
+        elif level == logging.DEBUG:
+            logging.debug(finalmsg)
+        elif level == logging.WARNING:
+            logging.warning(finalmsg)
+        else:
+            logging.info(finalmsg)
+
+    else :
+        print(msg, file=writter)
+
 def get_from_vars_or_args_or_environment(arg_pos, variable_name, vars, args):
     """Gets the argument from the position specified or gets the value from the table vars or gets the environment variable name passed.
 
@@ -137,7 +172,7 @@ def read_param_args(args):
         dictionary = { get_argkey(x) : get_argvalue(x) for x in args}
         if len(dictionary) != 0:
             has_passed_variables = True
-            print("Using variables")
+            log("Using variables")
             print_table(dictionary)
     return dictionary
 
@@ -148,15 +183,16 @@ def print_table(dictionary):
     length = len(dictionary)
     i = 0
     terminator = ', '
-    print("{", end='')
+    msg = "{"
     for key,value in dictionary.items():
         if ("PASSWORD" in key or "USER" in key):
             value = "*" * 12
         i += 1
         if (i == length) :
             terminator = ''
-        print(f"'{key}':'{value}'", end=terminator)
-    print("}")
+        msg += f"'{key}':'{value}'"+terminator
+    msg += "}"
+    log(msg)
 
 def expandvars(path, params, skip_escaped=False):
     """Expand environment variables of form $var and ${var}.
@@ -220,7 +256,7 @@ def log_on(user=None, password=None, account=None, database=None, warehouse=None
     """
     global error_code
     global opened_connections
-    print("arguments: ", sys.argv)
+    log("arguments: ",  sys.argv)
     # exclude arguments passed inline to the script
     args = [item for item in sys.argv if not item.startswith("--param-")]
     if (user is None):
@@ -257,7 +293,8 @@ def log_on(user=None, password=None, account=None, database=None, warehouse=None
             }
         )
     except Exception as e:
-        print(colored('*** Failure: logon failed :', 'red') + str(e))
+        print(e)
+        log(colored('*** Failure: logon failed :', 'red') + str(e), level = logging.ERROR, writter = sys.stderr)
         error_code = 333
         quit_application(error_code)
     if c:
@@ -268,7 +305,7 @@ def log_on(user=None, password=None, account=None, database=None, warehouse=None
 def at_exit_helpers():
     """Executes at the exit of the execution of the script
     """
-    print("Script done >>>>>>>>>>>>>>>>>>>>")
+    log("Script done >>>>>>>>>>>>>>>>>>>>")
     for c in opened_connections:
         if not c.is_closed():
             c.close()
@@ -286,7 +323,7 @@ def exception_hook(exctype, value, traceback):
     """
     traceback_formatted = traceback.format_exception(exctype, value, traceback)
     traceback_string = colored("*** Failure: " + "".join(traceback_formatted), 'red')
-    print(traceback_string, file=sys.stderr)
+    log(traceback_string, level = logging.ERROR, writter = sys.stderr)
     quit_application(1)
 
 def using(*argv):
@@ -338,15 +375,15 @@ def exec(sql_string, using=None, con=None):
     cur = con.cursor()
     try:
         error_code = 0
-        print(colored("Executing: ", 'blue') + colored(sql_string, 'cyan'))
+        log(colored("Executing: ") + colored(sql_string, 'cyan'))
         if ("$" in sql_string or "&" in sql_string):
-            print ("Expanding variables in SQL statement")
+            log ("Expanding variables in SQL statement")
             sql_string = expandvars(sql_string, passed_variables)
-            print (colored("Expanded string: ") + colored(sql_string, 'green'))
+            log (colored("Expanded string: ") + colored(sql_string, 'green'))
         if (using is not None):
                 #we need to change variables from {var} to %(format)
                 sql_string = re.sub(r'\{([^}]*)\}', r'%(\1)', sql_string)
-                print(f"using parameters {using}")
+                log(f"using parameters {using}")
         cur.execute(sql_string, params=using)
         activity_count = cur.rowcount
         if activity_count and activity_count >= 1:
@@ -361,13 +398,13 @@ def exec(sql_string, using=None, con=None):
         current_error_count = current_error_count + 1
         e = sys.exc_info()
         msg = colored("*** Failure running statement", 'red')
-        print(msg)
-        print(e)
+        log(msg, level= logging.ERROR, writter = sys.stderr)
+        log(e, level = logging.ERROR, writter = sys.stderr)
     finally:
         _previous_executed_sql = sql_string
         cur.close()
         if current_error_count >= max_errors:
-            print(colored(f'*** Failure: reached max error count {max_errors}', 'red'))
+            log(colored(f'*** Failure: reached max error count {max_errors}', 'red'), level = logging.ERROR)
             quit_application(1)
 
 def repeat_previous_sql_statement(con=None, n=1):
@@ -385,25 +422,25 @@ def repeat_previous_sql_statement(con=None, n=1):
         for rep in xrange(n):
             exec(_previous_executed_sql, con)
     else:
-        print("Warning: No previous SQL request.")
+        log("Warning: No previous SQL request.", level = logging.WARNING)
 
 def _print_result_set(cur):
     if (Export.expandedfilename is None):
         # if there is not export file set then print to console
-        print("Printing Result Set:")
-        print(','.join([col[0] for col in cur.description]))
+        log("Printing Result Set:")
+        log(','.join([col[0] for col in cur.description]))
         for row in cur:
-            print(','.join([str(val) for val in row]))
-        print()
+            log(','.join([str(val) for val in row]))
+        log("")
     else:
-        print(">>>>>> Exporting to " + Export.expandedfilename)
+        log(">>>>>> Exporting to ", Export.expandedfilename)
         reportdir = path.dirname(Export.expandedfilename)
         makedirs(reportdir, exist_ok=True)
         with open(Export.expandedfilename, 'a') as f:
             for row in cur:
                 allarenone = all(v is None for v in row)
                 if (allarenone):
-                    print("Row is 'None' it will not be exported")
+                    log("Row is 'None' it will not be exported")
                 else:
                     rowval = Export.separator.join([str(val) for val in row])
                     print(rowval, file=f)
@@ -423,7 +460,7 @@ def _handle_syntax_error(e, sql_string):
                     error_line = error_line[:column] + colored(error_line[column:], 'red')
                     error_fragment = error_fragment + f"{line:0>2d},{column:0>2d}:{error_line}\n"
                 last_line = line
-        print(error_fragment)
+        log(error_fragment, level = logging.ERROR)
     _handle_sql_error(e)
 
 def _handle_sql_error(e):
@@ -431,7 +468,7 @@ def _handle_sql_error(e):
     error_code = e.errno
     if error_code not in _severities_dictionary or _severities_dictionary[error_code] != 0:
         msg = colored("*** Failure " + str(e), 'red')
-        print(msg, file=sys.stderr)
+        log(msg, level = logging.ERROR, writter = sys.stderr)
         if error_code in _severities_dictionary:
             error_level = max(error_level, _severities_dictionary[error_code])
         else:
@@ -491,7 +528,7 @@ def exec_file(filename, con=None):
             for statement in statements:
                 exec(statement, con=con)
     else:
-        print(colored("*** Error: File does not exist", 'red'))
+        log(colored("*** Error: File does not exist", 'red'), level = logging.ERROR, writter = sys.stderr)
 
 def remark(arg):
     """Prints the argument.
@@ -499,7 +536,7 @@ def remark(arg):
         arg
             The argument to be printed
     """
-    print(arg)
+    log(arg)
 
 def get_error_position():
     """Gets the error position from the file using the
@@ -530,10 +567,10 @@ def quit_application(code=None):
         return
     quit_application_already_called = True
     code = code or error_level
-    print(colored(f"Error Code {code}", 'red'))
+    log(colored(f"Error Code {code}", 'red'), level = logging.ERROR, writter = sys.stderr)
     if code != 0:
-        stack_trace = get_error_position();
-        print(colored(stack_trace, 'red'))
+        stack_trace = get_error_position()
+        log(colored(stack_trace, 'red'), level = logging.ERROR, writter = sys.stderr)
     sys.exit(code)
 
 def import_file_to_temptable(filename, tempTableName, columnDefinition):
@@ -580,7 +617,7 @@ def file_exists_and_readable(filename):
 def exec_os(command):
     """Executes a command in the operative system.
     """
-    print(f"executing os command: {command}")
+    log(f"executing os command: {command}")
     return subprocess.getoutput(command)
 
 def fast_load(target_schema, filepath, stagename, target_table_name, con=None):
@@ -605,28 +642,28 @@ def fast_load(target_schema, filepath, stagename, target_table_name, con=None):
     stagename = expandvar(stagename)
     target_table_name = expandvar(target_table_name)
     exec(f""" USE SCHEMA {target_schema} """, con)
-    print(f"Putting file {filepath} into {stagename}...")
+    log(f"Putting file {filepath} into {stagename}...")
     exec(f"PUT file://{filepath} @{stagename} OVERWRITE = TRUE", con)
-    print(f"Done put file...ErrorCode {error_code}")
-    print(">>>Copying into...")
+    log(f"Done put file...ErrorCode {error_code}")
+    log(">>>Copying into...")
     exec(f"""
     COPY INTO {target_schema}.{target_table_name}
     FROM @{stagename}/{filename}
     FILE_FORMAT=(TYPE=CSV SKIP_HEADER=1)
     ON_ERROR=CONTINUE""", con)
-    print(f"<<<Done copying. ErrorCode {error_code}")
-    print(f">>>Creating temp table CTE_{target_table_name}")
+    log(f"<<<Done copying. ErrorCode {error_code}")
+    log(f">>>Creating temp table CTE_{target_table_name}")
     sql = f"CREATE TABLE {target_schema}.CTE_{target_table_name}  AS SELECT DISTINCT * FROM {target_schema}.{target_table_name}"
     exec(sql, con)
-    print(f"<<<Done creating temp table. ErrorCode {error_code}")
-    print(f">>>Droping old {target_table_name}")
+    log(f"<<<Done creating temp table. ErrorCode {error_code}")
+    log(f">>>Droping old {target_table_name}")
     sql = f"DROP TABLE {target_schema}.{target_table_name}"
     exec(sql, con)
-    print(f"<<<Done droping old table. ErrorCode {error_code}")
-    print(f">>>Renaming old CTE_{target_table_name}")
+    log(f"<<<Done droping old table. ErrorCode {error_code}")
+    log(f">>>Renaming old CTE_{target_table_name}")
     sql = f"ALTER TABLE {target_schema}.CTE_{target_table_name} RENAME TO {target_schema}.{target_table_name}"
     exec(sql, con)
-    print(f"<<<Done droping old table. ErrorCode {error_code}")
+    log(f"<<<Done droping old table. ErrorCode {error_code}")
 
 @deprecated(version='2.0', reason="This function will be removed soon, please use exec function instead")
 def execute_sql_statement(sql_string, con, using=None):
@@ -673,10 +710,10 @@ def simple_fast_load(con, target_schema, filepath, stagename, target_table_name)
 atexit.register(at_exit_helpers)
 
 def exception_hook(exctype, value, tback):
-    print(colored(f"*** Failure: {value}", 'red'), file=sys.stderr)
+    log(colored(f"*** Failure: {value}", 'red'), level = logging.ERROR, writter = sys.stderr)
     traceback_formatted = traceback.format_exception(exctype, value, tback)
     traceback_string = "".join(traceback_formatted)
-    print(traceback_string, file=sys.stderr)
+    log(traceback_string, level = logging.ERROR, writter = sys.stderr)
     quit_application(1)
 
 sys.excepthook = exception_hook
@@ -730,15 +767,15 @@ class Import:
             if (Import.expandedfilename is not None):
                 if (Import.reader is None):
                     read_obj = open(Import.expandedfilename, 'r')
-                    print(f">>>>>>>>> Importing from {Import.expandedfilename}")
+                    log(f">>>>>>>>> Importing from {Import.expandedfilename}")
                     if (stat(Import.expandedfilename).st_size == 0):
-                        print("Import file is empty")
+                        log("Import file is empty")
                         return
                     else:
                         # pass the file object to reader() to get the reader object
                         Import.reader = csv.reader(read_obj)
                 # read next row
-                print("Reading row")
+                log("Reading row")
                 row = next(Import.reader)
                 # row variable is a list that represents a row in csv
                 i = 0
@@ -747,10 +784,10 @@ class Import:
                     i += 1
         except StopIteration:
             Import.no_more_rows = True
-            print ("No more rows")
+            log ("No more rows")
         except Exception as e:
-            print (f"*** Failure importing {e}")
-        print("Done importing")
+            log (f"*** Failure importing {e}")
+        log("Done importing")
     def reset():
             Import.expandedfilename = None
             Import.separator = ' '
